clear 
set memory 3g
set more off
drop _all
local main "C:\The Reluctant Analyst\3&5"
cd "`main'"

*====================================================================
*===Identify analysts on the scale of 2, 3 and 4 (Sell, Hold and Buy)
*===input is the entire I/B/E/S Detail File downloaded in 2012
*====================================================================
use "`main'\DATA\Recommendations\Detail (entire)\c326a389b6272ec3.dta"
drop if usfirm==0 | ireccd=="" | ireccd=="0" | cusip=="" | amaskcd==0

gen month=mofd(actdats)
format month %tm
keep month emaskcd ireccd estimid

gen temp=real(ireccd)
drop ireccd 
rename temp ireccd 
replace ireccd=6-ireccd
label define ireccd_mask 5 "Strong Buy" 4 "Buy" 3 "Hold" 2 "Sell" 1 "Strong Sell"
label values ireccd ireccd_mask

*===drop if only one level ever issued
sort emaskcd 
by emaskcd: egen max_ireccd=max(ireccd)
by emaskcd: egen min_ireccd=min(ireccd)
drop if max_ireccd==min_ireccd
drop max_ireccd min_ireccd

*=== gen monthly max and min rec 
sort emaskcd month ireccd
by emaskcd month: drop if ireccd==ireccd[_n-1]
by emaskcd month: egen max_mon=max(ireccd)
by emaskcd month: egen min_mon=min(ireccd)
by emaskcd month: keep if _n==1
drop ireccd 

*=== drop if less than 20 monthes observed 
sort emaskcd 
by emaskcd: gen N=_N
drop if N<20 
drop N

*===min and max from a month onwards
gsort emaskcd -month 
quietly summarize month 
local min_mon=r(min)
local max_mon=r(max) 
gen max=.
gen min=.
forvalues mon=`min_mon' (1) `max_mon' {
quietly {
	by emaskcd: egen temp_max=max(max_mon) if month>=`mon'
	by emaskcd: egen temp_min=min(min_mon) if month>=`mon'

	replace max=temp_max if month==`mon'
	replace min=temp_min if month==`mon'
	drop temp_max temp_min 
	}
}

gen scale_234=1 if (max==4 & min==2) | (max==4 & min==3)
*===max=4 min=1 till 2010m10; 4 2 onwards: cannot distinguish
sort emaskcd month 
by emaskcd: gen temp=sum(scale_234)
replace scale_234=1 if scale_234==. & temp>0 
replace scale_234=0 if scale_234==.
drop temp 
save "`main'\scale_234.dta", replace 

*===included emaskcd (e.g. some emaskcd that have only issued one level were excluded)
sort emaskcd 
keep emaskcd
drop if emaskcd==emaskcd[_n-1] 
save "`main'\included_emaskcd.dta", replace 

*===3-point-scale brokers
use "`main'\scale_234.dta", clear 
tab scale_234
by emaskcd,sort: keep if scale_234==1
by emaskcd: keep if _n==1
keep emaskcd month
rename month scale_234_month
label var scale_234_month "on 3-point (2, 3 and 4) scale onwards"
save "`main'\scale_234.dta", replace 

*=======================================================================================
*===*===Identify analysts on the scale of 1, 3 and 5 (Strong sell, Hold, and Strong Buy)
*=======================================================================================
clear 
set more off
drop _all
local main "C:\Users\Chi\Desktop\The Pretty Good Analyst\3&5"
cd "`main'"

use "`main'\DATA\Recommendations\Detail (entire)\c326a389b6272ec3.dta"
drop if usfirm==0 | ireccd=="" | ireccd=="0" | cusip=="" | amaskcd==0

gen month=mofd(actdats)
format month %tm
keep month emaskcd ireccd estimid

gen temp=real(ireccd)
drop ireccd 
rename temp ireccd 
replace ireccd=6-ireccd
label define ireccd_mask 5 "Strong Buy" 4 "Buy" 3 "Hold" 2 "Sell" 1 "Strong Sell"
label values ireccd ireccd_mask

*===drop if only one level ever issued
sort emaskcd 
by emaskcd: egen max_ireccd=max(ireccd)
by emaskcd: egen min_ireccd=min(ireccd)
drop if max_ireccd==min_ireccd
drop max_ireccd min_ireccd

*=== gen monthly max, min and indicators for rec=4 and rec=2  
sort emaskcd month ireccd
by emaskcd month: drop if ireccd==ireccd[_n-1]
by emaskcd month: egen max_mon=max(ireccd)
by emaskcd month: egen min_mon=min(ireccd)
*===whether 2 or 4 ever issued in a month
egen temp=anyvalue(ireccd),v(4)
by emaskcd month: egen r4_mon=max(temp)
drop temp
replace r4_mon=0 if r4_mon==.
replace r4_mon=1 if r4_mon==4
egen temp=anyvalue(ireccd),v(2)
by emaskcd month: egen r2_mon=max(temp)
drop temp
replace r2_mon=0 if r2_mon==.
replace r2_mon=1 if r2_mon==2
by emaskcd month: keep if _n==1

drop ireccd 

*=== drop if less than 20 monthes observed 
sort emaskcd 
by emaskcd: gen N=_N
drop if N<20 
drop N

*===min and max from a month onwards; indicators for rec=4 and rec=2 onwards
gsort emaskcd -month 
quietly summarize month 
local min_mon=r(min)
local max_mon=r(max) 
gen max=.
gen min=.
gen r4=.
gen r2=.
forvalues mon=`min_mon' (1) `max_mon' {
quietly {
	by emaskcd: egen temp_max=max(max_mon) if month>=`mon'
	by emaskcd: egen temp_min=min(min_mon) if month>=`mon'
	by emaskcd: egen temp_r4=max(r4_mon) if month>=`mon'
	by emaskcd: egen temp_r2=max(r2_mon) if month>=`mon'

	replace max=temp_max if month==`mon'
	replace min=temp_min if month==`mon'
	replace r4=temp_r4 if month==`mon'
	replace r2=temp_r2 if month==`mon'
	drop temp_max temp_min temp_r4 temp_r2
	}
}

gen scale_135=1 if (max==5 & min==1 & r4!=1 & r2!=1) | (max==5 & min==3 & r4!=1 & r2!=1)
sort emaskcd month 
by emaskcd: gen temp=sum(scale_135)
replace scale_135=1 if scale_135==. & temp>0 
replace scale_135=0 if scale_135==.
drop temp 

tab scale_135
by emaskcd,sort: keep if scale_135==1
by emaskcd: keep if _n==1
keep emaskcd month
rename month scale_135_month
label var scale_135_month "on 3-point (1,3 and 5) scale onwards"
save "`main'\scale_135.dta", replace 
*===================================================================================
*===================================================================================
*===================================================================================

*===divided into three data sets: two on 3-point scale and one on 5-point scale 
local main "C:\The Reluctant Analyst\3&5"
use "`main'\DATA\Recommendations\Detail (entire)\c326a389b6272ec3.dta", clear
drop if usfirm==0 | ireccd=="" | ireccd=="0" | cusip=="" | amaskcd==0

gen month=mofd(actdats)
format month %tm

sort emaskcd
joinby emaskcd using "`main'\included_emaskcd.dta"
joinby emaskcd using "`main'\scale_234.dta", unmatched(master)
drop _merge
joinby emaskcd using "`main'\scale_135.dta", unmatched(master)
save "`main'\DATA\Recommendations\Detail (entire)\scale12345.dta", replace 

keep if month>=scale_234_month & scale_234_month!=.
drop  _merge scale_234_month scale_135_month
save "`main'\DATA\Recommendations\Detail (entire)\scale234.dta", replace 

use "`main'\DATA\Recommendations\Detail (entire)\scale12345.dta", clear 
keep if month>=scale_135_month & scale_135_month!=.
drop  _merge scale_234_month scale_135_month
save "`main'\DATA\Recommendations\Detail (entire)\scale135.dta", replace 

*===234+135=369 (We pool these two 3-tier populations into a single three-tier rating system)
use "`main'\DATA\Recommendations\Detail (entire)\scale234.dta", clear 
append using "`main'\DATA\Recommendations\Detail (entire)\scale135.dta"
replace ireccd="2" if ireccd=="1" 
replace ireccd="4" if ireccd=="5"
save "`main'\DATA\Recommendations\Detail (entire)\scale369.dta", replace 

use "`main'\DATA\Recommendations\Detail (entire)\scale12345.dta", clear 
* keep if month<scale_234_month | scale_234_month==. | month<scale_135_month  | scale_135_month==.
drop if (month>=scale_234_month & scale_234_month!=.) | (month>=scale_135_month & scale_135_month!=.) 
drop  _merge scale_234_month scale_135_month
save "`main'\DATA\Recommendations\Detail (entire)\scale12345.dta", replace 


* pooled sample containing all analysts on both three- and five-tier systems============================
* start with the pooled sample! ========================================================================
clear 
set more off
drop _all
local main "C:\The Reluctant Analyst\3&5"
cd "`main'"

set more off
*===we conservatively assume that an analyst who does not reiterate or revise a recommendation within 12 months has dropped coverage.
local fill_hole 12
*===We exclude analyst-firm pairs with fewer than 20 recommendations (including filled-in reiterations) over the entire sample period.
local min_obs 20

use "`main'\DATA\Recommendations\Detail (entire)\c326a389b6272ec3.dta", clear 
drop if usfirm==0 | ireccd=="" | ireccd=="0" | cusip=="" | amaskcd==0

gen temp=real(ireccd)
drop ireccd
rename temp ireccd

rename emaskcd bmaskcd
rename estimid broker

* use Activation Dates
rename actdats recdats

gen month=mofd(recdats)
format month %tm
gen quarter=qofd(recdats)
format quarter %tq
gen year=yofd(recdats)

drop if usfirm==0 | ireccd==. | cusip=="" | amaskcd==0
keep cusip amaskcd ireccd month quarter year bmaskcd broker cname recdats oftic
sort cusip amaskcd month 
*===for every analyst-firm pair, use the last recommendation of the month
drop if cusip==cusip[_n+1] & month==month[_n+1] & amaskcd==amaskcd[_n+1]

* firm_analyst indicates the firm-analyst pair
sort cusip amaskcd
egen firm_analyst=group(cusip amaskcd)

tsset  firm_analyst month
sort firm_analyst month
egen firm_analyst_month=group(firm_analyst month)
tsfill 

* if D==1, obs are filled
gen D=1 if  firm_analyst_month==.
local varlist "cusip cname oftic broker ireccd bmaskcd amaskcd firm_analyst_month"
foreach x in `varlist' {
	replace `x'=`x'[_n-1] if missing(`x')==1	
	}

replace quarter=int(month/3)
format quarter %tq
replace year=int(month/12)+1960 

sort firm_analyst month

by firm_analyst_month, sort: gen count_D=sum(D)
gen filled=1 if D==1
replace count_D=D if D==.
drop if count_D>`fill_hole' & missing(count_D)!=1

*===extend rec in the last month 
sort firm_analyst month
by firm_analyst: gen temp=1 if _n==_N
forvalues i=1 (1) `fill_hole' {
	expand 2 if temp==1, generate(temp2)
	replace month=month+`i' if temp2==1
	replace temp=0 if temp2==1
	drop temp2
}
sort firm_analyst month

*------------------------------------------------------------------------------------
* (sample period) 2003m1~2010m12
keep if 516<=month & month<=611
*------------------------------------------------------------------------------------

* drop if # of obs (after filling the holes) <= min_obs
by firm_analyst, sort: egen count=count(ireccd)
drop if count<=`min_obs'
drop count 

disp  "# of firm-analyst pair= " firm_analyst[_N]
sort cusip amaskcd month 
* regenerate firm_analyst indicator
drop firm_analyst
egen firm_analyst=group(cusip amaskcd)

*******************************************************************
sort firm_analyst month 
replace ireccd=6-ireccd
label define ireccd_mask 5 "Strong Buy" 4 "Buy" 3 "Hold" 2 "Sell" 1 "Strong Sell"
label values ireccd ireccd_mask

* generate file for estimation
outfile using "`main'\Anal_pooled_`fill_hole'_`min_obs'.txt", nolabel noquote wide replace


*===analysts on the three-tier system: 234 (S, H, B) & 135 (SS, H, SB)===============================================================================================================
foreach scale of numlist 234 135 {
use "`main'\Anal_pooled_`fill_hole'_`min_obs'.dta", clear 

rename bmaskcd emaskcd
rename broker estimid

sort emaskcd
joinby emaskcd using "`main'\scale_`scale'.dta", unmatched(master) 
keep if month>=scale_`scale'_month & scale_`scale'_month!=.

rename emaskcd bmaskcd 
rename estimid broker 

* regenerate firm_analyst indicator
drop firm_analyst
sort cusip amaskcd month
egen firm_analyst=group(cusip amaskcd)
sort firm_analyst month
drop count
by firm_analyst: gen count=_N
drop if count<=`min_obs'

if `scale'==234 {
replace ireccd=2 if ireccd==1
replace ireccd=4 if ireccd==5
}
if `scale'==135 {
replace ireccd=1 if ireccd==2
replace ireccd=5 if ireccd==4
}
outfile using "`main'\Anal_`scale'_`fill_hole'_`min_obs'.txt", nolabel noquote wide replace
}


*===analysts on the five-tier system: 12345 (SS, S, H, B, and SB)===============================================================================================================
use "`main'\Anal_pooled_`fill_hole'_`min_obs'.dta", clear 

rename bmaskcd emaskcd  
rename broker estimid  

sort emaskcd

joinby emaskcd using "`main'\scale_234.dta", unmatched(master) 
drop _merge 
joinby emaskcd using "`main'\scale_135.dta", unmatched(master) 
drop _merge 
* keep if (month>=scale_234_month & scale_234_month!=.) | (month>=scale_135_month & scale_135_month!=.) 
drop if (month>=scale_234_month & scale_234_month!=.) | (month>=scale_135_month & scale_135_month!=.) 

rename emaskcd bmaskcd 
rename estimid broker 

* regenerate firm_analyst indicator
drop firm_analyst
sort cusip amaskcd month
egen firm_analyst=group(cusip amaskcd)
sort firm_analyst month
drop count
by firm_analyst: gen count=_N
drop if count<=`min_obs'

sort cusip amaskcd month
* generate file for estimation
egen month_min=min(month)
replace month=month-month_min
format month %9.0g
outfile using "`main'\Anal_12345_`fill_hole'_`min_obs'.txt", nolabel noquote wide replace

